import { TypeKeys } from "web-utility";
import { HTTPClient, Context } from "koajax";
import { ValidationError } from "class-validator";
export type AbstractClass<I> = abstract new (...data: any[]) => I;
export type IDType = number | string;
export type DataObject = Record<string, any>;
export type NewData<T extends DataObject> = Partial<Omit<T, TypeKeys<T, DataObject> | TypeKeys<T, DataObject[]>> & {
    [K in TypeKeys<T, DataObject>]: IDType;
} & {
    [K in TypeKeys<T, DataObject[]>]: IDType[];
}>;
export type RESTClient = Pick<HTTPClient<Context>, 'baseURI' | 'options' | 'request' | 'get' | 'post' | 'put' | 'patch' | 'delete'>;
export abstract class BaseModel {
    downloading: number;
    uploading: number;
    clear(): this;
}
export abstract class BaseListModel<D extends DataObject> extends BaseModel {
    abstract client: RESTClient;
    abstract baseURI: string;
    currentOne: D;
    validity: InvalidMessage<D>;
    static createNested(parentId: IDType): BaseListModel<{}>;
    clear(): this;
    updateOne(data: NewData<D>, id?: IDType): Promise<D>;
    getOne(id: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
}
export function toggle<T extends BaseModel>(property: TypeKeys<T, boolean | number>): (target: any, key: string, meta: PropertyDescriptor) => void;
export function Input(): ParameterDecorator;
export type InvalidMessage<T> = Partial<Record<keyof T, ValidationError['constraints']>>;
export class InvalidError extends Error {
    static from<T>(state: InvalidMessage<T>): InvalidError;
}
export function validate<T extends BaseListModel<{}>>(): (target: any, key: string, meta: PropertyDescriptor) => void;
export interface PageData<D extends DataObject> {
    pageData: D[];
    totalCount?: number;
}
export abstract class ListModel<D extends DataObject, F extends NewData<D> = NewData<D>> extends BaseListModel<D> {
    pageIndex: number;
    pageSize: number;
    filter: F;
    noMore: boolean;
    totalCount: number;
    pageList: D[][];
    get currentPage(): D[];
    get allItems(): any[];
    constructor();
    clear(): this;
    turnTo(pageIndex: number, pageSize?: number): this;
    abstract loadPage(pageIndex: number, pageSize: number, filter: F): Promise<PageData<D>>;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    flushPage(pageIndex: number, pageSize: number, pageData: D[]): D[];
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
}
export function Buffer<D extends DataObject, F extends NewData<D> = NewData<D>, M extends AbstractClass<ListModel<D, F>> = AbstractClass<ListModel<D, F>>>(Super: M): (abstract new (...data: any[]) => {
    pendingList: ReturnType<ListModel<D, F>['loadPage']>[];
    clear(): any;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    noMore: boolean;
    totalCount: number;
    pageList: D[][];
    readonly currentPage: D[];
    readonly allItems: any[];
    turnTo(pageIndex: number, pageSize?: number): any;
    loadPage(pageIndex: number, pageSize: number, filter: F): Promise<PageData<D>>;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    flushPage(pageIndex: number, pageSize: number, pageData: D[]): D[];
    client: import("utility").RESTClient;
    baseURI: string;
    currentOne: D;
    validity: Partial<Record<keyof D, {
        [type: string]: string;
    }>>;
    updateOne(data: Partial<Omit<D, import("web-utility").TypeKeys<D, DataObject> | import("web-utility").TypeKeys<D, DataObject[]>> & { [K in import("web-utility").TypeKeys<D, DataObject>]: import("utility").IDType; } & { [K_1 in import("web-utility").TypeKeys<D, DataObject[]>]: import("utility").IDType[]; }>, id?: import("utility").IDType): Promise<D>;
    getOne(id: import("utility").IDType): Promise<D>;
    removeOne(id: import("utility").IDType): Promise<void>;
    downloading: number;
    uploading: number;
}) & M;
export function Stream<D extends DataObject, F extends NewData<D> = NewData<D>, M extends AbstractClass<ListModel<D, F>> = AbstractClass<ListModel<D, F>>>(Super: M): (abstract new (...data: any[]) => {
    baseURI: string;
    stream?: AsyncGenerator<D, void, any>;
    openStream(filter: F): AsyncGenerator<D, void, any>;
    clear(): any;
    loadPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
    }>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    noMore: boolean;
    totalCount: number;
    pageList: D[][];
    readonly currentPage: D[];
    readonly allItems: any[];
    turnTo(pageIndex: number, pageSize?: number): any;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    flushPage(pageIndex: number, pageSize: number, pageData: D[]): D[];
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    client: import("utility").RESTClient;
    currentOne: D;
    validity: Partial<Record<keyof D, {
        [type: string]: string;
    }>>;
    updateOne(data: Partial<Omit<D, import("web-utility").TypeKeys<D, DataObject> | import("web-utility").TypeKeys<D, DataObject[]>> & { [K in import("web-utility").TypeKeys<D, DataObject>]: import("utility").IDType; } & { [K_1 in import("web-utility").TypeKeys<D, DataObject[]>]: import("utility").IDType[]; }>, id?: import("utility").IDType): Promise<D>;
    getOne(id: import("utility").IDType): Promise<D>;
    removeOne(id: import("utility").IDType): Promise<void>;
    downloading: number;
    uploading: number;
}) & M;

//# sourceMappingURL=index.d.ts.map
